@@ -1,14 +1,14 @@
 #[allow(unreachable_pub)]
 pub use crate::builder::{Empty, TooSmallWorkspace};
 
-use core::{fmt, convert::From};
+use core::{convert::From, fmt};
 
 #[cfg(feature = "std")]
 use std::error::Error;
 
-
 /// Errors which could occur when using or creating a linear interpolation.
 #[derive(Debug, Copy, Clone)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub enum BezierError {
     /// Error returned if the generator does not contain any elements.
     Empty(Empty),
