@@ -1,16 +1,16 @@
 //! Builder module for linear interpolations.
 
-use core::ops::Mul;
+use super::error::LinearError;
+use super::{KnotElementInequality, Linear, TooFewElements};
+use crate::builder::{Type, Unknown, WithWeight, WithoutWeight};
+use crate::weights::{IntoWeight, Weighted, Weights};
+use crate::{DiscreteGenerator, Equidistant, Generator, Identity, Sorted, SortedGenerator};
 use core::marker::PhantomData;
+use core::ops::Mul;
+use num_traits::identities::Zero;
 use num_traits::real::Real;
 use num_traits::FromPrimitive;
-use num_traits::identities::Zero;
 use topology_traits::Merge;
-use crate::{Generator, DiscreteGenerator, SortedGenerator, Sorted, NotSorted, Equidistant, Identity};
-use crate::weights::{Weighted, Weights, IntoWeight};
-use crate::builder::{WithWeight,WithoutWeight, Type, Unknown};
-use super::Linear;
-use super::error::LinearError;
 
 /// Builder for linear interpolation.
 ///
@@ -29,10 +29,10 @@ use super::error::LinearError;
 /// #
 /// # fn main() -> Result<(), LinearError> {
 /// let linear = LinearDirector::new()
-///                 .elements([1.0,5.0,100.0])
+///                 .elements([1.0,5.0,100.0])?
 ///                 .equidistant::<f64>()
 ///                 .normalized()
-///                 .build()?;
+///                 .build();
 /// let results = [1.0,3.0,5.0,52.5,100.0];
 /// for (value,result) in linear.take(5).zip(results.iter().copied()){
 ///     assert_f64_near!(value, result);
@@ -60,7 +60,8 @@ use super::error::LinearError;
 /// [`equidistant()`]: LinearDirector::equidistant()
 /// [`equidistant_unchecked()`]: super::Linear::equidistant_unchecked()
 #[derive(Debug, Clone)]
-pub struct LinearDirector<K,E,F,W> {
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
+pub struct LinearDirector<K, E, F, W> {
     knots: K,
     elements: E,
     easing: F,
@@ -115,8 +116,9 @@ pub struct LinearDirector<K,E,F,W> {
 /// [`equidistant()`]: LinearBuilder::equidistant()
 /// [`equidistant_unchecked()`]: super::Linear::equidistant_unchecked()
 #[derive(Debug, Clone)]
-pub struct LinearBuilder<K,E,F,W> {
-    inner: Result<LinearDirector<K,E,F,W>,LinearError>,
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
+pub struct LinearBuilder<K, E, F, W> {
+    inner: Result<LinearDirector<K, E, F, W>, LinearError>,
 }
 
 impl Default for LinearDirector<Unknown, Unknown, Identity, Unknown> {
@@ -147,22 +149,35 @@ impl LinearBuilder<Unknown, Unknown, Identity, Unknown> {
     /// Create a new linear interpolation builder.
     pub const fn new() -> Self {
         LinearBuilder {
-            inner: Ok(LinearDirector::new())
+            inner: Ok(LinearDirector::new()),
         }
     }
 }
 
 impl<F> LinearDirector<Unknown, Unknown, F, Unknown> {
     /// Set the elements of the linear interpolation.
-    pub fn elements<E>(self, elements: E) -> LinearDirector<Unknown, E, F, WithoutWeight>
-    where E: DiscreteGenerator,
+    ///
+    /// # Errors
+    ///
+    /// Returns [`TooFewElements`] if not at least 2 elements are given.
+    ///
+    /// [`TooFewElements`]: super::error::LinearError
+    pub fn elements<E>(
+        self,
+        elements: E,
+    ) -> Result<LinearDirector<Unknown, E, F, WithoutWeight>, TooFewElements>
+    where
+        E: DiscreteGenerator,
     {
-        LinearDirector {
+        if elements.len() < 2 {
+            return Err(TooFewElements::new(elements.len()));
+        }
+        Ok(LinearDirector {
             knots: self.knots,
             elements,
             easing: self.easing,
             _phantom: PhantomData,
-        }
+        })
     }
 
     /// Set the elements and their weights for this interpolation.
@@ -195,7 +210,16 @@ impl<F> LinearDirector<Unknown, Unknown, F, Unknown> {
     /// #     Ok(())
     /// # }
     /// ```
-    pub fn elements_with_weights<G>(self, gen: G) -> LinearDirector<Unknown, Weights<G>, F, WithWeight>
+    ///
+    /// # Errors
+    ///
+    /// Returns [`TooFewElements`] if not at least 2 elements are given.
+    ///
+    /// [`TooFewElements`]: super::error::LinearError
+    pub fn elements_with_weights<G>(
+        self,
+        gen: G,
+    ) -> Result<LinearDirector<Unknown, Weights<G>, F, WithWeight>, TooFewElements>
     where
         G: DiscreteGenerator,
         G::Output: IntoWeight,
@@ -203,22 +227,28 @@ impl<F> LinearDirector<Unknown, Unknown, F, Unknown> {
             Mul<<G::Output as IntoWeight>::Weight, Output = <G::Output as IntoWeight>::Element>,
         <G::Output as IntoWeight>::Weight: Zero + Copy,
     {
-        LinearDirector {
+        if gen.len() < 2 {
+            return Err(TooFewElements::new(gen.len()));
+        }
+        Ok(LinearDirector {
             knots: self.knots,
             elements: Weights::new(gen),
             easing: self.easing,
             _phantom: PhantomData,
-        }
+        })
     }
 }
 
 impl<F> LinearBuilder<Unknown, Unknown, F, Unknown> {
     /// Set the elements of the linear interpolation.
     pub fn elements<E>(self, elements: E) -> LinearBuilder<Unknown, E, F, WithoutWeight>
-    where E: DiscreteGenerator,
+    where
+        E: DiscreteGenerator,
     {
         LinearBuilder {
-            inner: self.inner.map(|director| director.elements(elements))
+            inner: self
+                .inner
+                .and_then(|director| director.elements(elements).map_err(|err| err.into())),
         }
     }
 
@@ -252,7 +282,10 @@ impl<F> LinearBuilder<Unknown, Unknown, F, Unknown> {
     /// #     Ok(())
     /// # }
     /// ```
-    pub fn elements_with_weights<G>(self, gen: G) -> LinearBuilder<Unknown, Weights<G>, F, WithWeight>
+    pub fn elements_with_weights<G>(
+        self,
+        gen: G,
+    ) -> LinearBuilder<Unknown, Weights<G>, F, WithWeight>
     where
         G: DiscreteGenerator,
         G::Output: IntoWeight,
@@ -261,13 +294,16 @@ impl<F> LinearBuilder<Unknown, Unknown, F, Unknown> {
         <G::Output as IntoWeight>::Weight: Zero + Copy,
     {
         LinearBuilder {
-            inner: self.inner.map(|director| director.elements_with_weights(gen))
+            inner: self.inner.and_then(|director| {
+                director
+                    .elements_with_weights(gen)
+                    .map_err(|err| err.into())
+            }),
         }
     }
 }
 
-impl<E,F,W> LinearDirector<Unknown, E, F, W>
-{
+impl<E, F, W> LinearDirector<Unknown, E, F, W> {
     /// Set the knots of the interpolation.
     ///
     /// The amount of knots must be equal to the amount of elements.
@@ -278,12 +314,23 @@ impl<E,F,W> LinearDirector<Unknown, E, F, W>
     /// knots, consider using [`equidistant()`] instead.
     ///
     /// [`equidistant()`]: LinearDirector::equidistant()
-    pub fn knots<K>(self, knots: K) -> Result<LinearDirector<Sorted<K>,E, F, W>, NotSorted>
+    ///
+    /// # Errors
+    ///
+    /// Returns [`KnotElementInequality`] if the number of knots is not equal to the number of elements.
+    /// Returns [`NotSorted`] if the knots are not sorted such that they are increasing.
+    ///
+    /// [`KnotElementInequality`]: super::error::LinearError
+    /// [`NotSorted`]:  super::error::LinearError
+    pub fn knots<K>(self, knots: K) -> Result<LinearDirector<Sorted<K>, E, F, W>, LinearError>
     where
         E: DiscreteGenerator,
         K: DiscreteGenerator,
-        K::Output: PartialOrd
+        K::Output: PartialOrd,
     {
+        if self.elements.len() != knots.len() {
+            return Err(KnotElementInequality::new(self.elements.len(), knots.len()).into());
+        }
         Ok(LinearDirector {
             knots: Sorted::new(knots)?,
             elements: self.elements,
@@ -312,7 +359,7 @@ impl<E,F,W> LinearDirector<Unknown, E, F, W>
     /// [`domain()`]: LinearDirector::domain()
     /// [`normalized()`]: LinearDirector::normalized()
     /// [`distance()`]: LinearDirector::distance()
-    pub fn equidistant<R>(self) -> LinearDirector<Type<R>,E,F,W>{
+    pub fn equidistant<R>(self) -> LinearDirector<Type<R>, E, F, W> {
         LinearDirector {
             knots: Type::new(),
             elements: self.elements,
@@ -322,8 +369,7 @@ impl<E,F,W> LinearDirector<Unknown, E, F, W>
     }
 }
 
-impl<E,F,W> LinearBuilder<Unknown, E, F, W>
-{
+impl<E, F, W> LinearBuilder<Unknown, E, F, W> {
     /// Set the knots of the interpolation.
     ///
     /// The amount of knots must be equal to the amount of elements.
@@ -334,14 +380,14 @@ impl<E,F,W> LinearBuilder<Unknown, E, F, W>
     /// knots, consider using [`equidistant()`] instead.
     ///
     /// [`equidistant()`]: LinearBuilder::equidistant()
-    pub fn knots<K>(self, knots: K) -> LinearBuilder<Sorted<K>,E, F, W>
+    pub fn knots<K>(self, knots: K) -> LinearBuilder<Sorted<K>, E, F, W>
     where
         E: DiscreteGenerator,
         K: DiscreteGenerator,
-        K::Output: PartialOrd
+        K::Output: PartialOrd,
     {
         LinearBuilder {
-            inner: self.inner.and_then(|director| director.knots(knots).map_err(|err| err.into()))
+            inner: self.inner.and_then(|director| director.knots(knots)),
         }
     }
 
@@ -365,20 +411,20 @@ impl<E,F,W> LinearBuilder<Unknown, E, F, W>
     /// [`domain()`]: LinearBuilder::domain()
     /// [`normalized()`]: LinearBuilder::normalized()
     /// [`distance()`]: LinearBuilder::distance()
-    pub fn equidistant<R>(self) -> LinearBuilder<Type<R>,E,F,W>{
+    pub fn equidistant<R>(self) -> LinearBuilder<Type<R>, E, F, W> {
         LinearBuilder {
-            inner: self.inner.map(|director| director.equidistant())
+            inner: self.inner.map(|director| director.equidistant()),
         }
     }
 }
 
-impl<R,E,F,W> LinearDirector<Type<R>,E,F,W>
+impl<R, E, F, W> LinearDirector<Type<R>, E, F, W>
 where
     E: DiscreteGenerator,
     R: Real + FromPrimitive,
 {
     /// Set the domain of the interpolation.
-    pub fn domain(self, start: R, end: R) -> LinearDirector<Equidistant<R>,E,F,W>{
+    pub fn domain(self, start: R, end: R) -> LinearDirector<Equidistant<R>, E, F, W> {
         LinearDirector {
             knots: Equidistant::new(self.elements.len(), start, end),
             elements: self.elements,
@@ -388,7 +434,7 @@ where
     }
 
     /// Set the domain of the interpolation to be [0.0,1.0].
-    pub fn normalized(self) -> LinearDirector<Equidistant<R>,E,F,W>{
+    pub fn normalized(self) -> LinearDirector<Equidistant<R>, E, F, W> {
         LinearDirector {
             knots: Equidistant::normalized(self.elements.len()),
             elements: self.elements,
@@ -397,7 +443,7 @@ where
         }
     }
     /// Set the domain of the interpolation by defining the distance between the knots
-    pub fn distance(self, start: R, step: R) -> LinearDirector<Equidistant<R>,E,F,W>{
+    pub fn distance(self, start: R, step: R) -> LinearDirector<Equidistant<R>, E, F, W> {
         LinearDirector {
             knots: Equidistant::step(self.elements.len(), start, step),
             elements: self.elements,
@@ -407,35 +453,35 @@ where
     }
 }
 
-impl<R,E,F,W> LinearBuilder<Type<R>,E,F,W>
+impl<R, E, F, W> LinearBuilder<Type<R>, E, F, W>
 where
     E: DiscreteGenerator,
     R: Real + FromPrimitive,
 {
     /// Set the domain of the interpolation.
-    pub fn domain(self, start: R, end: R) -> LinearBuilder<Equidistant<R>,E,F,W>{
+    pub fn domain(self, start: R, end: R) -> LinearBuilder<Equidistant<R>, E, F, W> {
         LinearBuilder {
-            inner: self.inner.map(|director| director.domain(start,end))
+            inner: self.inner.map(|director| director.domain(start, end)),
         }
     }
 
     /// Set the domain of the interpolation to be [0.0,1.0].
-    pub fn normalized(self) -> LinearBuilder<Equidistant<R>,E,F,W>{
+    pub fn normalized(self) -> LinearBuilder<Equidistant<R>, E, F, W> {
         LinearBuilder {
-            inner: self.inner.map(|director| director.normalized())
+            inner: self.inner.map(|director| director.normalized()),
         }
     }
     /// Set the domain of the interpolation by defining the distance between the knots
-    pub fn distance(self, start: R, step: R) -> LinearBuilder<Equidistant<R>,E,F,W>{
+    pub fn distance(self, start: R, step: R) -> LinearBuilder<Equidistant<R>, E, F, W> {
         LinearBuilder {
-            inner: self.inner.map(|director| director.distance(start,step))
+            inner: self.inner.map(|director| director.distance(start, step)),
         }
     }
 }
 
-impl<K,E,F,W> LinearDirector<K,E,F,W>
+impl<K, E, F, W> LinearDirector<K, E, F, W>
 where
-    K: SortedGenerator
+    K: SortedGenerator,
 {
     /// Sets an easing function.
     ///
@@ -447,7 +493,7 @@ where
     /// See the [plateau example] for more information.
     ///
     /// [plateau example]: https://github.com/NicolasKlenert/enterpolation/blob/main/examples/plateaus.rs
-    pub fn easing<FF>(self, easing: FF) -> LinearDirector<K,E,FF,W>{
+    pub fn easing<FF>(self, easing: FF) -> LinearDirector<K, E, FF, W> {
         LinearDirector {
             knots: self.knots,
             elements: self.elements,
@@ -457,9 +503,9 @@ where
     }
 }
 
-impl<K,E,F,W> LinearBuilder<K,E,F,W>
+impl<K, E, F, W> LinearBuilder<K, E, F, W>
 where
-    K: SortedGenerator
+    K: SortedGenerator,
 {
     /// Sets an easing function.
     ///
@@ -471,43 +517,43 @@ where
     /// See the [plateau example] for more information.
     ///
     /// [plateau example]: https://github.com/NicolasKlenert/enterpolation/blob/main/examples/plateaus.rs
-    pub fn easing<FF>(self, easing: FF) -> LinearBuilder<K,E,FF,W>{
+    pub fn easing<FF>(self, easing: FF) -> LinearBuilder<K, E, FF, W> {
         LinearBuilder {
-            inner: self.inner.map(|director| director.easing(easing))
+            inner: self.inner.map(|director| director.easing(easing)),
         }
     }
 }
 
-impl<K,E,F> LinearDirector<K,E,F,WithoutWeight>
+impl<K, E, F> LinearDirector<K, E, F, WithoutWeight>
 where
     E: DiscreteGenerator,
     K: SortedGenerator,
     E::Output: Merge<K::Output>,
-    K::Output: Real
+    K::Output: Real,
 {
     /// Build a linear interpolation.
-    pub fn build(self) -> Result<Linear<K,E,F>,LinearError>{
-        Linear::new(self.elements, self.knots, self.easing)
+    pub fn build(self) -> Linear<K, E, F> {
+        Linear::new_unchecked(self.elements, self.knots, self.easing)
     }
 }
 
-impl<K,E,F> LinearBuilder<K,E,F,WithoutWeight>
+impl<K, E, F> LinearBuilder<K, E, F, WithoutWeight>
 where
     E: DiscreteGenerator,
     K: SortedGenerator,
     E::Output: Merge<K::Output>,
-    K::Output: Real
+    K::Output: Real,
 {
     /// Build a linear interpolation.
-    pub fn build(self) -> Result<Linear<K,E,F>,LinearError>{
+    pub fn build(self) -> Result<Linear<K, E, F>, LinearError> {
         match self.inner {
-            Err(err)=> Err(err),
-            Ok(director) => director.build()
+            Err(err) => Err(err),
+            Ok(director) => Ok(director.build()),
         }
     }
 }
 
-impl<K,G,F> LinearDirector<K,Weights<G>,F,WithWeight>
+impl<K, G, F> LinearDirector<K, Weights<G>, F, WithWeight>
 where
     K: SortedGenerator,
     K::Output: Real + Copy,
@@ -516,13 +562,16 @@ where
     <Weights<G> as Generator<usize>>::Output: Merge<K::Output>,
 {
     /// Build a weighted linear interpolation.
-    pub fn build(self) -> Result<WeightedLinear<K,G,F>, LinearError>
-    {
-        Ok(Weighted::new(Linear::new(self.elements, self.knots, self.easing)?))
+    pub fn build(self) -> WeightedLinear<K, G, F> {
+        Weighted::new(Linear::new_unchecked(
+            self.elements,
+            self.knots,
+            self.easing,
+        ))
     }
 }
 
-impl<K,G,F> LinearBuilder<K,Weights<G>,F,WithWeight>
+impl<K, G, F> LinearBuilder<K, Weights<G>, F, WithWeight>
 where
     K: SortedGenerator,
     K::Output: Real + Copy,
@@ -531,49 +580,89 @@ where
     <Weights<G> as Generator<usize>>::Output: Merge<K::Output>,
 {
     /// Build a weighted linear interpolation.
-    pub fn build(self) -> Result<WeightedLinear<K,G,F>, LinearError>
-    {
+    pub fn build(self) -> Result<WeightedLinear<K, G, F>, LinearError> {
         match self.inner {
-            Err(err)=> Err(err),
-            Ok(director) => director.build()
+            Err(err) => Err(err),
+            Ok(director) => Ok(director.build()),
         }
     }
 }
 
 /// Type alias for weighted linear interpolations
-type WeightedLinear<K,G,F> = Weighted<Linear<K,Weights<G>,F>>;
+type WeightedLinear<K, G, F> = Weighted<Linear<K, Weights<G>, F>>;
 
 #[cfg(test)]
 mod test {
     use super::LinearBuilder;
     // Homogeneous for creating Homogeneous, Generator for using .stack()
-    use crate::{weights::Homogeneous, Generator};
+    use crate::{linear::LinearDirector, weights::Homogeneous, Generator};
     #[test]
     fn building_weights() {
         LinearBuilder::new()
-            .elements_with_weights([(1.0,1.0),(2.0,2.0),(3.0,0.0)])
+            .elements_with_weights([(1.0, 1.0), (2.0, 2.0), (3.0, 0.0)])
             .equidistant::<f64>()
             .normalized()
-            .build().unwrap();
+            .build()
+            .unwrap();
         LinearBuilder::new()
-            .elements_with_weights([1.0,2.0,3.0].stack([1.0,2.0,0.0]))
+            .elements_with_weights([1.0, 2.0, 3.0].stack([1.0, 2.0, 0.0]))
             .equidistant::<f64>()
             .normalized()
-            .build().unwrap();
+            .build()
+            .unwrap();
         LinearBuilder::new()
             .elements_with_weights([
                 Homogeneous::new(1.0),
                 Homogeneous::weighted_unchecked(2.0, 2.0),
-                Homogeneous::infinity(3.0)])
-            .knots([1.0,2.0,3.0])
-            .build().unwrap();
+                Homogeneous::infinity(3.0),
+            ])
+            .knots([1.0, 2.0, 3.0])
+            .build()
+            .unwrap();
         LinearBuilder::new()
-            .elements([0.1,0.2,0.3])
+            .elements([0.1, 0.2, 0.3])
             .equidistant::<f64>()
             .normalized()
-            .build().unwrap();
-        assert!(LinearBuilder::new().elements::<[f64;0]>([]).knots::<[f64;0]>([]).build().is_err());
-        assert!(LinearBuilder::new().elements([1.0]).knots([1.0]).build().is_err());
-        assert!(LinearBuilder::new().elements([1.0,2.0]).knots([1.0,2.0,3.0]).build().is_err());
+            .build()
+            .unwrap();
+    }
+
+    #[test]
+    fn builder_errors() {
+        assert!(LinearBuilder::new()
+            .elements::<[f64; 0]>([])
+            .knots::<[f64; 0]>([])
+            .build()
+            .is_err());
+        assert!(LinearBuilder::new()
+            .elements([1.0])
+            .knots([1.0])
+            .build()
+            .is_err());
+        assert!(LinearBuilder::new()
+            .elements([1.0, 2.0])
+            .knots([1.0, 2.0, 3.0])
+            .build()
+            .is_err());
+    }
+
+    #[test]
+    fn director_errors() {
+        assert!(LinearDirector::new().elements([0.0]).is_err());
+        assert!(LinearDirector::new()
+            .elements([0.0, 1.0])
+            .unwrap()
+            .knots([1.0])
+            .is_err());
+        assert!(LinearDirector::new()
+            .elements([1.0, 2.0])
+            .unwrap()
+            .knots([1.0, 2.0, 3.0])
+            .is_err());
+        assert!(LinearDirector::new()
+            .elements([1.0, 2.0])
+            .unwrap()
+            .knots([1.0, 2.0])
+            .is_ok());
     }
 }
